import { Mapper } from '@mobicoop/ddd-library';
import { Injectable } from '@nestjs/common';
import { ConfigurationEntity } from './core/domain/configuration.entity';
import {
  ConfigurationReadModel,
  ConfigurationWriteModel,
} from './infrastructure/configuration.repository';
import { ConfigurationResponseDto } from './interface/dtos/configuration.response.dto';

/**
 * Mapper constructs objects that are used in different layers:
 * Record is an object that is stored in a database,
 * Entity is an object that is used in application domain layer,
 * and a ResponseDTO is an object returned to a user (usually as json).
 */

@Injectable()
export class ConfigurationMapper
  implements
    Mapper<
      ConfigurationEntity,
      ConfigurationReadModel,
      ConfigurationWriteModel,
      ConfigurationResponseDto
    >
{
  toPersistence = (entity: ConfigurationEntity): ConfigurationWriteModel => {
    const copy = entity.getProps();
    const record: ConfigurationWriteModel = {
      uuid: entity.id,
      domain: copy.domain,
      key: copy.key,
      value: copy.value,
    };
    return record;
  };

  toDomain = (record: ConfigurationReadModel): ConfigurationEntity => {
    const entity = new ConfigurationEntity({
      id: record.uuid,
      createdAt: new Date(record.createdAt),
      updatedAt: new Date(record.updatedAt),
      props: {
        domain: record.domain,
        key: record.key,
        value: record.value,
      },
    });
    return entity;
  };

  toResponse = (entity: ConfigurationEntity): ConfigurationResponseDto => {
    const props = entity.getProps();
    const response = new ConfigurationResponseDto(entity);
    response.domain = props.domain;
    response.key = props.key;
    response.value = props.value;
    return response;
  };
}
