import { Module, Provider } from '@nestjs/common';
import { CqrsModule } from '@nestjs/cqrs';
import { GetConfigurationGrpcController } from './interface/grpc-controllers/get-configuration.grpc.controller';
import { SetConfigurationGrpcController } from './interface/grpc-controllers/set-configuration.grpc.controller';
import { DeleteConfigurationGrpcController } from './interface/grpc-controllers/delete-configuration.grpc.controller';
import { PropagateConfigurationsGrpcController } from './interface/grpc-controllers/propagate-configurations.grpc.controller';
import { PublishMessageWhenConfigurationIsSetDomainEventHandler } from './core/application/event-handlers/publish-message-when-configuration-is-set.domain-event-handler';
import { PublishMessageWhenConfigurationIsDeletedDomainEventHandler } from './core/application/event-handlers/publish-message-when-configuration-is-deleted.domain-event-handler';
import { SetConfigurationService } from './core/application/commands/set-configuration/set-configuration.service';
import { DeleteConfigurationService } from './core/application/commands/delete-configuration/delete-configuration.service';
import { GetConfigurationQueryHandler } from './core/application/queries/get-configuration/get-configuration.query-handler';
import { ConfigurationMapper } from './configuration.mapper';
import {
  CONFIGURATION_MESSAGE_PUBLISHER,
  CONFIGURATION_REPOSITORY,
} from './configuration.di-tokens';
import { ConfigurationRepository } from './infrastructure/configuration.repository';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { PrismaService } from './infrastructure/prisma.service';
import { PropagateConfigurationsService } from './core/application/commands/propagate-configurations/propagate-configurations.service';

const grpcControllers = [
  GetConfigurationGrpcController,
  SetConfigurationGrpcController,
  DeleteConfigurationGrpcController,
  PropagateConfigurationsGrpcController,
];

const eventHandlers: Provider[] = [
  PublishMessageWhenConfigurationIsSetDomainEventHandler,
  PublishMessageWhenConfigurationIsDeletedDomainEventHandler,
];

const commandHandlers: Provider[] = [
  SetConfigurationService,
  DeleteConfigurationService,
  PropagateConfigurationsService,
];

const queryHandlers: Provider[] = [GetConfigurationQueryHandler];

const mappers: Provider[] = [ConfigurationMapper];

const repositories: Provider[] = [
  {
    provide: CONFIGURATION_REPOSITORY,
    useClass: ConfigurationRepository,
  },
];

const messagePublishers: Provider[] = [
  {
    provide: CONFIGURATION_MESSAGE_PUBLISHER,
    useExisting: MessageBrokerPublisher,
  },
];
const orms: Provider[] = [PrismaService];

@Module({
  imports: [CqrsModule],
  controllers: [...grpcControllers],
  providers: [
    ...eventHandlers,
    ...commandHandlers,
    ...queryHandlers,
    ...mappers,
    ...repositories,
    ...messagePublishers,
    ...orms,
  ],
  exports: [PrismaService, ConfigurationMapper, CONFIGURATION_REPOSITORY],
})
export class ConfigurationModule {}
