import { Inject } from '@nestjs/common';
import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { DeleteConfigurationCommand } from './delete-configuration.command';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { ConfigurationRepositoryPort } from '../../ports/configuration.repository.port';
import { ConfigurationEntity } from '@modules/configuration/core/domain/configuration.entity';

@CommandHandler(DeleteConfigurationCommand)
export class DeleteConfigurationService implements ICommandHandler {
  constructor(
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly configurationRepository: ConfigurationRepositoryPort,
  ) {}

  async execute(command: DeleteConfigurationCommand): Promise<boolean> {
    const configuration: ConfigurationEntity =
      await this.configurationRepository.findOne({
        domain: command.domain,
        key: command.key,
      });
    configuration.delete();
    const isDeleted: boolean =
      await this.configurationRepository.delete(configuration);
    return isDeleted;
  }
}
