import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { Inject } from '@nestjs/common';
import {
  CONFIGURATION_MESSAGE_PUBLISHER,
  CONFIGURATION_REPOSITORY,
} from '@modules/configuration/configuration.di-tokens';
import { ConfigurationRepositoryPort } from '../../ports/configuration.repository.port';
import { PropagateConfigurationsCommand } from './propagate-configurations.command';
import { ConfigurationEntity } from '@modules/configuration/core/domain/configuration.entity';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { CONFIGURATION_PROPAGATED_ROUTING_KEY } from '@src/app.constants';
import { ConfigurationMapper } from '@modules/configuration/configuration.mapper';

@CommandHandler(PropagateConfigurationsCommand)
export class PropagateConfigurationsService implements ICommandHandler {
  constructor(
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly repository: ConfigurationRepositoryPort,
    @Inject(CONFIGURATION_MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
    private readonly configurationMapper: ConfigurationMapper,
  ) {}

  async execute(): Promise<void> {
    const configurationItems: ConfigurationEntity[] =
      await this.repository.findAll({});
    this.messagePublisher.publish(
      CONFIGURATION_PROPAGATED_ROUTING_KEY,
      JSON.stringify(
        configurationItems.map((configuration: ConfigurationEntity) =>
          this.configurationMapper.toResponse(configuration),
        ),
      ),
    );
  }
}
