import { CommandHandler, ICommandHandler } from '@nestjs/cqrs';
import { Inject } from '@nestjs/common';
import { AggregateID, NotFoundException } from '@mobicoop/ddd-library';
import { SetConfigurationCommand } from './set-configuration.command';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { ConfigurationRepositoryPort } from '../../ports/configuration.repository.port';
import { ConfigurationEntity } from '@modules/configuration/core/domain/configuration.entity';

@CommandHandler(SetConfigurationCommand)
export class SetConfigurationService implements ICommandHandler {
  constructor(
    @Inject(CONFIGURATION_REPOSITORY)
    private readonly repository: ConfigurationRepositoryPort,
  ) {}

  async execute(command: SetConfigurationCommand): Promise<AggregateID> {
    try {
      const existingConfiguration: ConfigurationEntity =
        await this.repository.findOne({
          domain: command.domain,
          key: command.key,
        });
      existingConfiguration.update(command);
      await this.repository.update(
        existingConfiguration.id,
        existingConfiguration,
      );
      return existingConfiguration.id;
    } catch (error: any) {
      if (error instanceof NotFoundException) {
        try {
          const newConfiguration = ConfigurationEntity.create(command);
          await this.repository.insert(newConfiguration);
          return newConfiguration.id;
        } catch (error: any) {
          throw error;
        }
      }
      throw error;
    }
  }
}
