import { Inject, Injectable } from '@nestjs/common';
import { OnEvent } from '@nestjs/event-emitter';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { CONFIGURATION_MESSAGE_PUBLISHER } from '@modules/configuration/configuration.di-tokens';
import { ConfigurationDeletedDomainEvent } from '../../domain/events/configuration-deleted.domain-event';
import { ConfigurationDeletedIntegrationEvent } from '../events/configuration-deleted.integration-event';
import { CONFIGURATION_DELETED_ROUTING_KEY } from '@src/app.constants';

@Injectable()
export class PublishMessageWhenConfigurationIsDeletedDomainEventHandler {
  constructor(
    @Inject(CONFIGURATION_MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
  ) {}

  @OnEvent(ConfigurationDeletedDomainEvent.name, {
    async: true,
    promisify: true,
  })
  async handle(event: ConfigurationDeletedDomainEvent): Promise<any> {
    const configurationDeletedIntegrationEvent =
      new ConfigurationDeletedIntegrationEvent({
        id: event.aggregateId,
        domain: event.domain,
        key: event.key,
        metadata: event.metadata,
      });
    this.messagePublisher.publish(
      CONFIGURATION_DELETED_ROUTING_KEY,
      JSON.stringify(configurationDeletedIntegrationEvent),
    );
  }
}
