import { Inject, Injectable } from '@nestjs/common';
import { OnEvent } from '@nestjs/event-emitter';
import { MessagePublisherPort } from '@mobicoop/ddd-library';
import { ConfigurationSetIntegrationEvent } from '../events/configuration-set.integration-event';
import { CONFIGURATION_MESSAGE_PUBLISHER } from '@modules/configuration/configuration.di-tokens';
import { ConfigurationSetDomainEvent } from '../../domain/events/configuration-set.domain-event';
import { CONFIGURATION_SET_ROUTING_KEY } from '@src/app.constants';

@Injectable()
export class PublishMessageWhenConfigurationIsSetDomainEventHandler {
  constructor(
    @Inject(CONFIGURATION_MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
  ) {}

  @OnEvent(ConfigurationSetDomainEvent.name, { async: true, promisify: true })
  async handle(event: ConfigurationSetDomainEvent): Promise<any> {
    const configurationSetIntegrationEvent =
      new ConfigurationSetIntegrationEvent({
        id: event.aggregateId,
        domain: event.domain,
        key: event.key,
        value: event.value,
        metadata: event.metadata,
      });
    this.messagePublisher.publish(
      CONFIGURATION_SET_ROUTING_KEY,
      JSON.stringify(configurationSetIntegrationEvent),
    );
  }
}
