import { AggregateRoot, AggregateID } from '@mobicoop/ddd-library';
import { v4 } from 'uuid';
import {
  ConfigurationProps,
  CreateConfigurationProps,
  UpdateConfigurationProps,
} from './configuration.types';
import { ConfigurationSetDomainEvent } from './events/configuration-set.domain-event';
import { ConfigurationDeletedDomainEvent } from './events/configuration-deleted.domain-event';

export class ConfigurationEntity extends AggregateRoot<ConfigurationProps> {
  protected readonly _id: AggregateID;

  static create = (create: CreateConfigurationProps): ConfigurationEntity => {
    const id = v4();
    const props: ConfigurationProps = { ...create };
    const configuration = new ConfigurationEntity({ id, props });
    configuration.addEvent(
      new ConfigurationSetDomainEvent({
        aggregateId: id,
        domain: props.domain,
        key: props.key,
        value: props.value,
      }),
    );
    return configuration;
  };

  update(props: UpdateConfigurationProps): void {
    this.props.value = props.value;
    this.addEvent(
      new ConfigurationSetDomainEvent({
        aggregateId: this._id,
        domain: this.props.domain,
        key: this.props.key,
        value: props.value,
      }),
    );
  }

  delete(): void {
    this.addEvent(
      new ConfigurationDeletedDomainEvent({
        aggregateId: this.id,
        domain: this.props.domain,
        key: this.props.key,
      }),
    );
  }

  validate(): void {
    // entity business rules validation to protect it's invariant before saving entity to a database
  }
}
