import { Inject, Injectable, Logger } from '@nestjs/common';
import { EventEmitter2 } from '@nestjs/event-emitter';
import {
  LoggerBase,
  MessagePublisherPort,
  PrismaRepositoryBase,
} from '@mobicoop/ddd-library';
import { SERVICE_NAME } from '@src/app.constants';
import { ConfigurationEntity } from '../core/domain/configuration.entity';
import { ConfigurationRepositoryPort } from '../core/application/ports/configuration.repository.port';
import { PrismaService } from './prisma.service';
import { CONFIGURATION_MESSAGE_PUBLISHER } from '../configuration.di-tokens';
import { ConfigurationMapper } from '../configuration.mapper';

export type ConfigurationBaseModel = {
  uuid: string;
  domain: string;
  key: string;
  value: string;
};

export type ConfigurationReadModel = ConfigurationBaseModel & {
  createdAt: Date;
  updatedAt: Date;
};

export type ConfigurationWriteModel = ConfigurationBaseModel;

/**
 *  Repository is used for retrieving/saving domain entities
 * */
@Injectable()
export class ConfigurationRepository
  extends PrismaRepositoryBase<
    ConfigurationEntity,
    ConfigurationReadModel,
    ConfigurationWriteModel
  >
  implements ConfigurationRepositoryPort
{
  constructor(
    prisma: PrismaService,
    mapper: ConfigurationMapper,
    eventEmitter: EventEmitter2,
    @Inject(CONFIGURATION_MESSAGE_PUBLISHER)
    protected readonly messagePublisher: MessagePublisherPort,
  ) {
    super(
      prisma.configuration,
      prisma,
      mapper,
      eventEmitter,
      new LoggerBase({
        logger: new Logger(ConfigurationRepository.name),
        domain: SERVICE_NAME,
        messagePublisher,
      }),
    );
  }
}
