import { ConfigurationEntity } from '@modules/configuration/core/domain/configuration.entity';
import {
  CreateConfigurationProps,
  Domain,
  UpdateConfigurationProps,
} from '@modules/configuration/core/domain/configuration.types';
import { ConfigurationDeletedDomainEvent } from '@modules/configuration/core/domain/events/configuration-deleted.domain-event';
import { ConfigurationSetDomainEvent } from '@modules/configuration/core/domain/events/configuration-set.domain-event';

const createConfigurationProps: CreateConfigurationProps = {
  domain: Domain.AD,
  key: 'seatsProposed',
  value: '3',
};

const updateConfigurationProps: UpdateConfigurationProps = {
  value: '2',
};

describe('Configuration entity create', () => {
  it('should create a new configuration entity', async () => {
    const configurationEntity: ConfigurationEntity = ConfigurationEntity.create(
      createConfigurationProps,
    );
    expect(configurationEntity.id.length).toBe(36);
    expect(configurationEntity.getProps().value).toBe('3');
    expect(configurationEntity.domainEvents.length).toBe(1);
    expect(configurationEntity.domainEvents[0]).toBeInstanceOf(
      ConfigurationSetDomainEvent,
    );
  });
});

describe('Configuration entity update', () => {
  it('should update a configuration entity', async () => {
    const configurationEntity: ConfigurationEntity = ConfigurationEntity.create(
      createConfigurationProps,
    );
    configurationEntity.update(updateConfigurationProps);
    expect(configurationEntity.getProps().value).toBe('2');
    // 2 events because ConfigurationEntity.create sends a ConfigurationSetDomainEvent
    expect(configurationEntity.domainEvents.length).toBe(2);
    expect(configurationEntity.domainEvents[1]).toBeInstanceOf(
      ConfigurationSetDomainEvent,
    );
  });
});

describe('Configuration entity delete', () => {
  it('should delete a configuration entity', async () => {
    const configurationEntity: ConfigurationEntity = ConfigurationEntity.create(
      createConfigurationProps,
    );
    configurationEntity.delete();
    // 2 events because ConfigurationEntity.create sends a ConfigurationSetDomainEvent
    expect(configurationEntity.domainEvents.length).toBe(2);
    expect(configurationEntity.domainEvents[1]).toBeInstanceOf(
      ConfigurationDeletedDomainEvent,
    );
  });
});
