import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { DeleteConfigurationCommand } from '@modules/configuration/core/application/commands/delete-configuration/delete-configuration.command';
import { DeleteConfigurationService } from '@modules/configuration/core/application/commands/delete-configuration/delete-configuration.service';
import { Domain } from '@modules/configuration/core/domain/configuration.types';
import { DeleteConfigurationRequestDto } from '@modules/configuration/interface/grpc-controllers/dtos/delete-configuration.request.dto';
import { Test, TestingModule } from '@nestjs/testing';

const deleteConfigurationRequest: DeleteConfigurationRequestDto = {
  domain: Domain.AD,
  key: 'seatsProposed',
};

const mockConfigurationEntity = {
  delete: jest.fn(),
};

const mockConfigurationRepository = {
  findOne: jest.fn().mockImplementation(() => mockConfigurationEntity),
  delete: jest.fn().mockImplementationOnce(() => true),
};

describe('Delete Configuration Service', () => {
  let deleteConfigurationService: DeleteConfigurationService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CONFIGURATION_REPOSITORY,
          useValue: mockConfigurationRepository,
        },
        DeleteConfigurationService,
      ],
    }).compile();

    deleteConfigurationService = module.get<DeleteConfigurationService>(
      DeleteConfigurationService,
    );
  });

  it('should be defined', () => {
    expect(deleteConfigurationService).toBeDefined();
  });

  describe('execution', () => {
    const deleteConfigurationCommand = new DeleteConfigurationCommand(
      deleteConfigurationRequest,
    );
    it('should delete a configuration item', async () => {
      const result: boolean = await deleteConfigurationService.execute(
        deleteConfigurationCommand,
      );
      expect(result).toBeTruthy();
    });
  });
});
