import { CONFIGURATION_MESSAGE_PUBLISHER } from '@modules/configuration/configuration.di-tokens';
import { PublishMessageWhenConfigurationIsDeletedDomainEventHandler } from '@modules/configuration/core/application/event-handlers/publish-message-when-configuration-is-deleted.domain-event-handler';
import { Domain } from '@modules/configuration/core/domain/configuration.types';
import { ConfigurationDeletedDomainEvent } from '@modules/configuration/core/domain/events/configuration-deleted.domain-event';
import { Test, TestingModule } from '@nestjs/testing';
import { CONFIGURATION_DELETED_ROUTING_KEY } from '@src/app.constants';

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

describe('Publish message when configuration is deleted domain event handler', () => {
  let publishMessageWhenConfigurationIsDeletedDomainEventHandler: PublishMessageWhenConfigurationIsDeletedDomainEventHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CONFIGURATION_MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
        PublishMessageWhenConfigurationIsDeletedDomainEventHandler,
      ],
    }).compile();

    publishMessageWhenConfigurationIsDeletedDomainEventHandler =
      module.get<PublishMessageWhenConfigurationIsDeletedDomainEventHandler>(
        PublishMessageWhenConfigurationIsDeletedDomainEventHandler,
      );
  });

  it('should publish a message', () => {
    jest.spyOn(mockMessagePublisher, 'publish');
    const configurationDeletedDomainEvent: ConfigurationDeletedDomainEvent = {
      id: 'some-domain-event-id',
      domain: Domain.AD,
      key: 'seatsProposed',
      aggregateId: 'some-aggregate-id',
      metadata: {
        timestamp: new Date('2023-06-28T05:00:00Z').getTime(),
        correlationId: 'some-correlation-id',
      },
    };
    publishMessageWhenConfigurationIsDeletedDomainEventHandler.handle(
      configurationDeletedDomainEvent,
    );
    expect(
      publishMessageWhenConfigurationIsDeletedDomainEventHandler,
    ).toBeDefined();
    expect(mockMessagePublisher.publish).toHaveBeenCalledTimes(1);
    expect(mockMessagePublisher.publish).toHaveBeenCalledWith(
      CONFIGURATION_DELETED_ROUTING_KEY,
      '{"id":"some-aggregate-id","metadata":{"correlationId":"some-correlation-id","timestamp":1687928400000},"domain":"AD","key":"seatsProposed"}',
    );
  });
});
