import { CONFIGURATION_MESSAGE_PUBLISHER } from '@modules/configuration/configuration.di-tokens';
import { PublishMessageWhenConfigurationIsSetDomainEventHandler } from '@modules/configuration/core/application/event-handlers/publish-message-when-configuration-is-set.domain-event-handler';
import { Domain } from '@modules/configuration/core/domain/configuration.types';
import { ConfigurationSetDomainEvent } from '@modules/configuration/core/domain/events/configuration-set.domain-event';
import { Test, TestingModule } from '@nestjs/testing';
import { CONFIGURATION_SET_ROUTING_KEY } from '@src/app.constants';

const mockMessagePublisher = {
  publish: jest.fn().mockImplementation(),
};

describe('Publish message when configuration is set domain event handler', () => {
  let publishMessageWhenConfigurationIsSetDomainEventHandler: PublishMessageWhenConfigurationIsSetDomainEventHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CONFIGURATION_MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
        PublishMessageWhenConfigurationIsSetDomainEventHandler,
      ],
    }).compile();

    publishMessageWhenConfigurationIsSetDomainEventHandler =
      module.get<PublishMessageWhenConfigurationIsSetDomainEventHandler>(
        PublishMessageWhenConfigurationIsSetDomainEventHandler,
      );
  });

  it('should publish a message', () => {
    jest.spyOn(mockMessagePublisher, 'publish');
    const configurationSetDomainEvent: ConfigurationSetDomainEvent = {
      id: 'some-domain-event-id',
      aggregateId: 'some-aggregate-id',
      domain: Domain.AD,
      key: 'seatsProposed',
      value: '3',
      metadata: {
        timestamp: new Date('2023-06-28T05:00:00Z').getTime(),
        correlationId: 'some-correlation-id',
      },
    };
    publishMessageWhenConfigurationIsSetDomainEventHandler.handle(
      configurationSetDomainEvent,
    );
    expect(
      publishMessageWhenConfigurationIsSetDomainEventHandler,
    ).toBeDefined();
    expect(mockMessagePublisher.publish).toHaveBeenCalledTimes(1);
    expect(mockMessagePublisher.publish).toHaveBeenCalledWith(
      CONFIGURATION_SET_ROUTING_KEY,
      '{"id":"some-aggregate-id","metadata":{"correlationId":"some-correlation-id","timestamp":1687928400000},"domain":"AD","key":"seatsProposed","value":"3"}',
    );
  });
});
