import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { PropagateConfigurationsGrpcController } from '@modules/configuration/interface/grpc-controllers/propagate-configurations.grpc.controller';
import { CommandBus } from '@nestjs/cqrs';
import { RpcException } from '@nestjs/microservices';
import { Test, TestingModule } from '@nestjs/testing';

const mockCommandBus = {
  execute: jest
    .fn()
    .mockImplementationOnce(() => {})
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

describe('Propagate Configurations Grpc Controller', () => {
  let propagateConfigurationsGrpcController: PropagateConfigurationsGrpcController;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CommandBus,
          useValue: mockCommandBus,
        },
        PropagateConfigurationsGrpcController,
      ],
    }).compile();

    propagateConfigurationsGrpcController =
      module.get<PropagateConfigurationsGrpcController>(
        PropagateConfigurationsGrpcController,
      );
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(propagateConfigurationsGrpcController).toBeDefined();
  });

  it('should propagate configuration items', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    await propagateConfigurationsGrpcController.propagate();
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });

  it('should throw a generic RpcException', async () => {
    jest.spyOn(mockCommandBus, 'execute');
    try {
      await propagateConfigurationsGrpcController.propagate();
    } catch (e: any) {
      expect(e).toBeInstanceOf(RpcException);
      expect(e.error.code).toBe(RpcExceptionCode.UNKNOWN);
    }
    expect(mockCommandBus.execute).toHaveBeenCalledTimes(1);
  });
});
